/************************************************************************
	List Row Object
*************************************************************************/	

const LIST_ROW_HEIGHT = 20;
const LIST_ROW_HOFFSET = 10;
const LIST_ROW_HIGHLIGHT_OPACITY = 200;
const LIST_ROW_INACTIVE_OPACITY = 130;
const LIST_ROW_DRAGGER_SRC = 'Resources/Images/dragger.png';
const LIST_ROW_DRAGGER_OPACITY = 190;
const LIST_ROW_DRAG_SENSITIVITY = 2;

function ListRow(id, title, pane)
{
	this.id = id;
	this.pane = pane;
	this.selected = false;
	this.reorderable = true;
	this.dragging = false;
	this.dragged = false;
	this.clicked = { x: null, y: null };
	
	this.frame = new Frame();
	pane.list.appendChild(this.frame);
	
	// Row background
	this.background = new Image();
	this.background.src = "Resources/Images/row.png";
	this.background.hOffset = 0;
	this.background.row = this;
	this.background.tracking = "rectangle";
	this.background.onContextMenu = function(){pane.onRowContextMenu(this.row)};
	this.frame.appendChild(this.background);

	// Row highlight
	this.highlight = new Image();
	this.highlight.src = "Resources/Images/highlight.png";
	this.highlight.hOffset = 0;
	this.highlight.row = this;
	this.highlight.tracking = "rectangle";
	this.highlight.onMouseEnter = ListRow.prototype.focus;
	this.highlight.onMouseMove = ListRow.prototype.onMouseMove;
	this.highlight.onMouseExit = ListRow.prototype.loseFocus;
	this.highlight.onContextMenu = ListRow.prototype.onRowContextMenu;
	this.frame.appendChild(this.highlight);
	
	// Row title
	this.title = new Text();
	if(title) this.title.data = title;
	this.title.hOffset = LIST_ROW_HOFFSET;
	this.title.vOffset = LIST_ROW_HEIGHT - 6;
	this.title.size = 11;
	this.title.color = "#FFFFFF";
	this.title.shadow = blackShadow;
	this.title.truncation = "end";
	this.title.row = this;
	this.title.onContextMenu = function(){pane.onRowContextMenu(this.row)};
	this.frame.appendChild(this.title);
	
	// Row dragger
	this.dragger = new Image();
	this.dragger.src = LIST_ROW_DRAGGER_SRC;
	this.dragger.hOffset = 0;
	this.dragger.opacity = 1;
	this.dragger.row = this;
	this.dragger.onClick = ListRow.prototype.onClick;
	this.dragger.onMouseDown = ListRow.prototype.onMouseDown;
	this.dragger.onMouseDrag = ListRow.prototype.onMouseDrag;
	this.dragger.onMouseUp = ListRow.prototype.onMouseUp;
	pane.dragList.appendChild(this.dragger);
	
	this.watch("vOffset", ListRow.prototype.vOffsetWatch);
	this.watch("width", ListRow.prototype.widthWatch);

	this.width = pane.width;
	this.setID(id, true); // Set initial ID
}

ListRow.prototype.realign = function() {
	this.frame.vOffset = this.vOffset;
	this.dragger.hOffset = 0;
	this.dragger.vOffset = this.dragger.parentNode.convertPointFromWindow(this.frame.parentNode.convertPointToWindow(0, this.vOffset)).y;
}

ListRow.prototype.vOffsetWatch = function(prop, oldval, newval)
{
	this.vOffset = newval;
	this.realign();
	return newval;
}

ListRow.prototype.widthWatch = function(prop, oldval, newval)
{
	this.dragger.width = this.highlight.width = this.background.width = newval;	
	this.title.width = newval - 30;			
	return newval;
}

ListRow.prototype.setTitle = function(title)
{
	this.title.data = title;
}

// Set the "active" state of the row
ListRow.prototype.setActive = function(active, animate)
{
	if(animate)
	{
		if(this.title.fadeAnimation) this.title.fadeAnimation.kill();
		this.title.fadeAnimation = new FadeAnimation(this.title, active ? 255 : LIST_ROW_INACTIVE_OPACITY, 200, animator.kEaseOut);
		animator.start(this.title.fadeAnimation);
	}
	else this.title.opacity = active ? 255 : LIST_ROW_INACTIVE_OPACITY;
}

ListRow.prototype.setSelected = function(selected)
{
	this.selected = selected;
	if(this.highlight.fadeIn) this.highlight.fadeIn.kill();
	if(this.highlight.fadeOut) this.highlight.fadeOut.kill();

	this.highlight.opacity = selected ? LIST_ROW_HIGHLIGHT_OPACITY : 1;
}

// Rearrange row position within the list
ListRow.prototype.setID = function(id, move)
{
	this.id = id;	
	if(move) this.vOffset = LIST_ROW_HEIGHT*id;
	this.updateAppearance();
}

ListRow.prototype.updateAppearance = function()
{
	this.title.font = preferences.font.value;
	this.highlight.opacity = this.selected ? LIST_ROW_HIGHLIGHT_OPACITY : 1;
}

// Apply context menu items to the row's objects
ListRow.prototype.setContextMenu = function(items)
{
	this.highlight.contextMenuItems = this.title.contextMenuItems = this.background.contextMenuItems = items;
}

ListRow.prototype.setTooltip = function(tooltip)
{
	this.highlight.tooltip = tooltip;
}

// Focus row on mouse over
ListRow.prototype.focus = function()
{
	var row = this.row;
	if (!row.pane.dragging) {
		if (!overlay.opacity) {
			row.highlight.opacity = LIST_ROW_HIGHLIGHT_OPACITY;
		}
		row.pane.onMouseEnterRow(row);
	}
}

ListRow.prototype.onMouseMove = function()
{
	var row = this.row;
	if (!row.pane.dragging) {
		row.pane.onMouseMoveOverRow(row);
	}
}

// Defocus row on mouse exit
ListRow.prototype.loseFocus = function()
{
	var row = this.row;
	if (row != row.pane.selectedRow) {
		row.highlight.opacity = 1;
	}
	row.pane.onMouseExitRow(row);
}

ListRow.prototype.remove = function()
{
	if(this.highlight.fadeIn) this.highlight.fadeIn.kill();
	if(this.highlight.fadeOut) this.highlight.fadeOut.kill();
	
	this.background.removeFromSuperview();
	this.background = null;
	
	this.highlight.removeFromSuperview();
	this.highlight = null;
	
	this.title.removeFromSuperview();
	this.title = null;
	
	this.dragger.removeFromSuperview();
	this.dragger = null;
	
	this.frame.removeFromSuperview();
	this.frame = null;
}

// Used to trigger pane's onRowSelected "event"
ListRow.prototype.onClick = function()
{
	var row = this.row;
	if (!row.dragged) {
		row.pane.onRowClicked(row);
		if((row.pane.exType && row != row.pane.selectedRow) || !row.pane.exType)
		{
			if(row.pane.exType) row.pane.selectRow(row, true);
			else row.pane.onRowSelected(row, true);
		}
	}
}

ListRow.prototype.onMouseDown = function() {
	var row = this.row;
	
	if (row.pane.reorderable && row.reorderable) {
		row.pane.dragging = row.dragging = true;
		row.dragged = false;
		// create a snapshot of the row to be dragged
		var tmpSrc = system.widgetDataFolder + "/tmpDragger.png";
		
		// clear any old src on the row to prevent file locking issues
		row.dragger.src = LIST_ROW_DRAGGER_SRC;
		row.frame.saveImageToFile(tmpSrc, 'png');
		
		row.dragger.src = tmpSrc;
		row.dragger.orderAbove();
	}
		
	row.clicked.x = system.event.x;
	row.clicked.y = system.event.y;
}

ListRow.prototype.onMouseUp = function() {
	var row = this.row;
	if (row.pane.reorderable && row.reorderable) {
		if (row.dragged) {
			// handle new drag location
			var newRowId = ListRow.calculateNewRow(row);
			if (newRowId > row.id) {
				// recalculate rowID, based on the row being deleted
				newRowId--;
			}
			if (newRowId == row.id) {
				// if the row is not moving, don't do anything
			} else {
				var pane = row.pane;
				pane.reorderRow(row, newRowId);
			}
			
		}
		//  return the dragger and whatnot to its correct position
		row.realign();
		row.dragger.opacity = 1;
		row.pane.dragPreview.visible = false;
		row.pane.dragging = row.dragging = false;
		row.dragger.src = LIST_ROW_DRAGGER_SRC;
	}
}

// Used to reorder rows
ListRow.prototype.onMouseDrag = function()
{
	var row = this.row;
	if (row.pane.reorderable && row.reorderable) {
		if (!row.dragged && (Math.abs(system.event.x - row.clicked.x) + Math.abs(system.event.y - row.clicked.y)) > LIST_ROW_DRAG_SENSITIVITY) {
			// start dragging if we've crossed the sensitivity threshold
			row.dragger.opacity = LIST_ROW_DRAGGER_OPACITY;
			row.pane.dragPreview.visible = true;
			row.dragged = true;
			ListRow.prototype.loseFocus.call(this);
		}
		
		if (row.dragged) {
			// move the dragger image
			var p = row.dragger.parentNode.convertPointFromWindow(system.event.hOffset, system.event.vOffset);
			row.dragger.hOffset = p.x - row.clicked.x;
			row.dragger.vOffset = p.y - row.clicked.y;
			
			// move the dragPreview bar
			var newRowId = ListRow.calculateNewRow(row);
			row.pane.dragPreview.vOffset = newRowId*LIST_ROW_HEIGHT;
		}
	}
}


ListRow.prototype.toString = function() {
	return 'ListRow [' + this.id + ']: ' + this.title.data;
}



ListRow.calculateNewRow = function(row) {
	// calculate closest row, based on center of row
	// only applicable to a row currently being dragged
	return Math.max(0, (Math.min(row.pane.rows.length, Math.round((row.dragger.vOffset + LIST_ROW_HEIGHT/2) / LIST_ROW_HEIGHT))));
}
